package PC20050922;

import java.util.*;
import ProSW.*;

public abstract class Correntista {
	private final String nome, cognome, codFis;
	private String telefono;
	private Indirizzo indirizzo;
	private int monteOreMensile;
	
	private static final int SALDO_INIZIALE = 10;
	
	public Correntista(String n, String c, String cf, Indirizzo i, String tel, int more) throws EccezionePrecondizioni {
		if (n == null) throw new EccezionePrecondizioni("Il nome  obbligatorio");
		nome = n;
		
		if (c == null) throw new EccezionePrecondizioni("Il cognome  obbligatorio");
		cognome = c;
	
		if (cf == null) throw new EccezionePrecondizioni("Il codice fiscale  obbligatorio");
		if (cf.length() != 16) throw new EccezionePrecondizioni("Il codice fiscale deve essere di 16 caratteri");
		codFis = cf;

		setIndirizzo(i);
		
		setTelefono(tel);
		setMonteOreMensile(more);
	}


	public String getNome() { return nome; }
	public String getCognome() { return cognome; }
	
	public String getTelefono() { return telefono; }
	public void setTelefono(String tel) throws EccezionePrecondizioni { 
		if (tel == null) throw new EccezionePrecondizioni("Il telefono  null!");
		telefono = tel;
	}

	public Indirizzo getIndirizzo() { return (Indirizzo)indirizzo.clone(); }
	public void setIndirizzo(Indirizzo ind) throws EccezionePrecondizioni { 
		if (ind == null) throw new EccezionePrecondizioni("L'indirizzo  null!");
		indirizzo = (Indirizzo)ind.clone();
	}


	public int getMonteOreMensile() { return monteOreMensile; }
	public void setMonteOreMensile(int more) throws EccezionePrecondizioni { 
		if (more <= 0) throw new EccezionePrecondizioni("Il monte ore mensile deve essere positivo");
		monteOreMensile = more;
	}

	// Operazioni di classe
	public double gradoOnestaAutovalutazione() {
		double m = 0;
		Iterator<TipoLinkPossiede> it = this.linkPossiede.iterator();
		while (it.hasNext()) {
			TipoLinkPossiede tlp = it.next();
			try {
				m += tlp.getGradoAutovalutazione() - mediaFeedback(tlp.getAbilita());
			} catch(EccezionePrecondizioni e) {
				// Non puo' accadere
			  }
		}
		m = m / linkPossiede.size();
		
		if (m <= 0) return 10;
		else {
			if (10 - m < 0) throw new InternalError("Errore: grado onesta' autov. < 0!"); // Non dovrebbe mai accadere
			return 10-m;
		}
	}
	
	
	public double mediaFeedback(Abilita a) throws EccezionePrecondizioni {
		TipoLinkPossiede test = new TipoLinkPossiede(this, a, 6);
		if (!linkPossiede.contains(test)) throw new EccezionePrecondizioni("L'abilit non  posseduta dal correntista");
		// Si noti come il valore 6 per l'attributo del link non influenza in alcun modo Set.contains(), 
		// perche' nella classe TipoLinkPossiede abbiamo fatto overriding di equals().
		// Alternativamente, posso scandire linkPossiede alla ricerca di un link l 
		// tale che l.getCorrentista()==this && l.getAbilita()==a
		
		double somma = 0;
		int cont = 0;
		Iterator<TipoLinkOfferente> it = linkOfferente.iterator();
		while (it.hasNext()) {
			Scambio s = it.next().getScambio();
			if (s.getAbilitaRichiesta() == a) {
				cont ++;
				somma += s.getFeedback();
			}
		}
		
		if (cont == 0) return 10;
		else return somma / cont;
	}
		
	
	
	public double saldo() {
		DataOra adesso = new DataOra();
		double result = SALDO_INIZIALE;

		Iterator<TipoLinkOfferente> itOff = this.linkOfferente.iterator();
		while (itOff.hasNext()) {
			Scambio s = itOff.next().getScambio();
			if (s.getMomento().primaDiOUgualeA(adesso))
				result += s.valore();
		}
		
		Iterator<TipoLinkRichiedente> itRich = this.linkRichiedente.iterator();
		while (itRich.hasNext()) {
			Scambio s = itRich.next().getScambio();
			if (s.getMomento().primaDiOUgualeA(adesso))
				result -= s.valore();
		}
		return result;
	}
	
	public abstract double valoreOra();
	
	public double monteOreResiduo(int mese, int anno) throws EccezionePrecondizioni {
		if (mese < 1 || mese > 12) throw new EccezionePrecondizioni("Il mese deve essere in 1..12");
		if (anno <= 0) throw new EccezionePrecondizioni("L'anno deve essere positivo");
		
		double result = monteOreMensile;
		
		Iterator<TipoLinkOfferente> it = linkOfferente.iterator();
		while (it.hasNext()) {
			Scambio s = it.next().getScambio();
			DataOra momento = s.getMomento();
			if (momento.getData().getAnno() == anno && momento.getData().getMese() == mese) {
				result -= s.getDurataOre();
			}
		}
		
		// Non dovrebbe mai capitare, solo per robustezza e per semplificare il debug
		if (result < 0) throw new InternalError("Errore: il monte ore  negativo!");
		
		return result;
	}

	// Implementazione semplificata (solo debug)
	public String toString() {
		return nome;
	}
	
	// Gestione associazioni: lasciata per esercizio...
	
} //:~
