package PC20050922;

import java.util.*;
import ProSW.*;

public final class RicercaMiglioriOfferenti {

	private RicercaMiglioriOfferenti() {}
	
	public static Set<Correntista> ricercaMiglioriOfferenti(Correntista richiedente, DataOra quando, double durata, Abilita a, int autovMin) throws EccezionePrecondizioni {
		if (richiedente == null) throw new EccezionePrecondizioni("Il richiedente non puo' essere null");
		if (quando == null) throw new EccezionePrecondizioni("Il momento non puo' essere null");
		if (durata <= 0) throw new EccezionePrecondizioni("La durata deve essere positiva");
		if (a == null) throw new EccezionePrecondizioni("L'abilita' non puo' essere null");
		
		if (autovMin < 6 || autovMin > 10) throw new EccezionePrecondizioni("L'autovalutazione minima deve essere in 6..10");
		
		Set<Correntista> result = new HashSet<Correntista>();
		double distanzaMin = -1;
		
		Iterator<TipoLinkPossiede> it = a.getInsLinkPossiede().iterator();
		while (it.hasNext()) {
			TipoLinkPossiede tlp = it.next();
			if (tlp.getGradoAutovalutazione() < autovMin) continue;
			Correntista c = tlp.getCorrentista();
			if (c.monteOreResiduo( quando.getData().getMese(), quando.getData().getAnno() ) < durata) continue;
			
			// Il correntista  disponibile:  tra i piu' vicini?
			double distanza = richiedente.getIndirizzo().distanzaDa(c.getIndirizzo());
			if (distanzaMin < 0 || distanza < distanzaMin) {
				// La condizione distanzaMin < 0 serve per gestire il primo inserimento, quando distanzaMin e' settata al valore -1
				
				// E' migliore degli altri: svuoto l'insieme e lo inserisco. Aggiorno distanzaMin.
				result = new HashSet<Correntista>(); // oppure result.clear();
				result.add(c);
				distanzaMin = distanza;
			}
			else if (distanzaMin == distanza) { 
				// E' tra i migliori finora trovati
				result.add(c); 
			}
		}
		
		return result;
	}
		
} //:~
